import {
  DatabaseErrorException,
  NotFoundException,
  RpcExceptionCode,
  RpcValidationPipe,
} from '@mobicoop/ddd-library';
import { PauseAdCommand } from '@modules/ad/core/application/commands/pause-ad/pause-ad.command';
import { Controller, UsePipes } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { GRPC_SERVICE_NAME } from '@src/app.constants';
import { PauseAdRequestDto } from './dtos/pause-ad.request.dto';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class PauseAdGrpcController {
  constructor(private readonly commandBus: CommandBus) {}

  @GrpcMethod(GRPC_SERVICE_NAME, 'Pause')
  async pause(data: PauseAdRequestDto): Promise<void> {
    try {
      await this.commandBus.execute(new PauseAdCommand(data));
    } catch (error: any) {
      if (error instanceof NotFoundException)
        throw new RpcException({
          code: RpcExceptionCode.NOT_FOUND,
          message: error.message,
        });
      if (error instanceof DatabaseErrorException)
        throw new RpcException({
          code: RpcExceptionCode.INTERNAL,
          message: error.message,
        });
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: error.message,
      });
    }
  }
}
